import { Observable } from 'rxjs';
const defaultDeserializer = ({ data }) => {
    try {
        return JSON.parse(data);
    }
    catch {
        return data;
    }
};
// tslint:disable-next-line rxjs-no-subclass
export class TransportObservable extends Observable {
    constructor(transport, { deserializer = defaultDeserializer, openObserver }) {
        super((observer) => {
            const handleCloseEvent = () => observer.complete();
            const handleErrorEvent = ((({ error }) => (error === undefined ? observer.error(new Error('Unknown Error')) : observer.error(error))));
            const handleMessageEvent = ((event) => observer.next(deserializer(event)));
            const handleOpenEvent = () => {
                if (openObserver !== undefined) {
                    openObserver.next();
                }
            };
            transport.addEventListener('close', handleCloseEvent);
            transport.addEventListener('error', handleErrorEvent);
            transport.addEventListener('message', handleMessageEvent);
            transport.addEventListener('open', handleOpenEvent);
            return () => {
                transport.removeEventListener('close', handleCloseEvent);
                transport.removeEventListener('error', handleErrorEvent);
                transport.removeEventListener('message', handleMessageEvent);
                transport.removeEventListener('open', handleOpenEvent);
            };
        });
    }
}
//# sourceMappingURL=transport-observable.js.map