import { Observable, Observer } from 'rxjs';
import { AnonymousSubject } from 'rxjs/internal/Subject'; // tslint:disable-line rxjs-no-compat no-submodule-imports rxjs-no-internal
import { filter, map } from 'rxjs/operators';
import { IRemoteSubject } from '../interfaces';
import { TGetTypedKeysFunction, TStringifyableJsonObject, TStringifyableJsonValue } from '../types';

export class MaskedSubject<
        T extends TStringifyableJsonValue,
        U extends TStringifyableJsonObject & { message: T },
        V extends TStringifyableJsonObject | U
    >
    extends AnonymousSubject<T>
    implements IRemoteSubject<T>
{
    // tslint:disable-line max-line-length
    private _mask: Partial<Omit<U, 'message'>>;

    // tslint:disable-next-line rxjs-no-exposed-subjects
    private _maskableSubject: IRemoteSubject<V>;

    // tslint:disable-next-line max-line-length rxjs-no-exposed-subjects
    constructor(getTypedKeys: TGetTypedKeysFunction, mask: Partial<Omit<U, 'message'>>, maskableSubject: IRemoteSubject<V>) {
        const destination: Observer<T> = {
            complete: maskableSubject.complete,
            error: maskableSubject.error,
            next: (value) => this.send(value)
        };

        const stringifiedValues = getTypedKeys(mask).map((key) => [key, JSON.stringify(mask[key])] as const);

        const source: Observable<T> = (<Observable<TStringifyableJsonObject | U>>maskableSubject).pipe(
            filter((message): message is U => stringifiedValues.every(([key, value]) => value === JSON.stringify(message[key]))),
            map(({ message }) => message)
        );

        super(destination, source);

        this._mask = mask;
        this._maskableSubject = maskableSubject;
    }

    public close(): void {
        this._maskableSubject.close();
    }

    public send(value: T): Promise<void> {
        return this._maskableSubject.send(<V>{ ...this._mask, message: value });
    }
}
